"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.postCSSSerializer = exports.babelStyleSerializer = exports.testStyleSerializer = exports.serializeStyles = void 0;
const types_1 = require("@babel/types");
/**
 * The Default Serializer which separates the styles into { style, media }
 */
function serializeStyles(styleRecord, replacer) {
    const media = {};
    let hasPlatform = false;
    let hasPlatformColor = false;
    const styles = Object.fromEntries(Object.entries(styleRecord).flatMap(([key, value]) => {
        return value.map((style) => {
            var _a;
            let newKey = key;
            if ("atRules" in style) {
                const { atRules, ...rest } = style;
                (_a = media[key]) !== null && _a !== void 0 ? _a : (media[key] = []);
                newKey = `${key}.${media[key].length}`;
                media[key].push(atRules);
                style = rest;
            }
            if (replacer) {
                style = Object.fromEntries(Object.entries(style).map(([k, v]) => {
                    if (typeof v === "string") {
                        hasPlatform || (hasPlatform = v.includes("platform("));
                        hasPlatformColor || (hasPlatformColor = v.includes("platformColor("));
                    }
                    return replacer(k, v);
                }));
            }
            return [newKey, style];
        });
    }));
    return {
        styles,
        media,
        hasPlatform,
        hasPlatformColor,
    };
}
exports.serializeStyles = serializeStyles;
/**
 * Used by Jest
 *
 * Replaces platform function strings with known values
 */
function testStyleSerializer(styleRecord) {
    return serializeStyles(styleRecord, (key, value) => {
        if (value === "styleSheet(hairlineWidth)") {
            return [key, 1];
        }
        return [key, value];
    });
}
exports.testStyleSerializer = testStyleSerializer;
/**
 * Used by Babel
 *
 * Replaces platform function strings with actual platform functions
 */
function babelStyleSerializer(styleRecord) {
    return serializeStyles(styleRecord, babelReplacer);
}
exports.babelStyleSerializer = babelStyleSerializer;
function babelReplacer(key, value) {
    if (typeof value !== "string") {
        return [key, value];
    }
    if (value === "styleSheet(hairlineWidth)") {
        return [
            key,
            (0, types_1.memberExpression)((0, types_1.identifier)("RNStyleSheet"), (0, types_1.identifier)("hairlineWidth")),
        ];
    }
    if (value.startsWith("platformColor(")) {
        const result = value.match(/platformColor\((.+)\)/);
        if (!result)
            return [key, (0, types_1.identifier)("undefined")];
        const variables = result[1]
            .split(/[ ,]+/)
            .filter(Boolean)
            .map((v) => (0, types_1.stringLiteral)(v));
        return [key, (0, types_1.callExpression)((0, types_1.identifier)("RNPlatformColor"), variables)];
    }
    if (value.startsWith("platform(")) {
        const result = value.match(/platform\((.+)\)/);
        if (!result)
            return [key, (0, types_1.identifier)("undefined")];
        const props = result[1]
            .trim()
            .split(/\s+/)
            .map((a) => {
            // Edge case:
            //   platform(android:platformColor(@android:color/holo_blue_bright))
            // Sometimes the value can has a colon, so we need to collect all values
            // and join them
            let [platform, ...values] = a.split(":");
            if (!values) {
                values = [platform];
                platform = "default";
            }
            const [key, value] = babelReplacer(platform, `${values.join("")}`);
            if (typeof value === "object" && (0, types_1.isExpression)(value)) {
                return (0, types_1.objectProperty)((0, types_1.identifier)(key), value);
            }
            else if (typeof value === "string") {
                return (0, types_1.objectProperty)((0, types_1.identifier)(key), (0, types_1.stringLiteral)(value));
            }
            else {
                throw new TypeError("Shouldn't reach here");
            }
        });
        return [
            key,
            (0, types_1.callExpression)((0, types_1.memberExpression)((0, types_1.identifier)("RNPlatform"), (0, types_1.identifier)("select")), [(0, types_1.objectExpression)(props)]),
        ];
    }
    return [key, value];
}
/**
 * Used by Post CSS
 *
 * Replaces platform function strings with actual platform functions
 */
function postCSSSerializer(styleRecord) {
    const { styles, media, ...rest } = serializeStyles(styleRecord, postCSSReplacer);
    return {
        style: JSON.stringify(styles).replaceAll(/"?!!!"?/g, ""),
        media: JSON.stringify(media),
        ...rest,
    };
}
exports.postCSSSerializer = postCSSSerializer;
function postCSSReplacer(key, value) {
    if (typeof value !== "string") {
        return [key, value];
    }
    if (value === "styleSheet(hairlineWidth)") {
        return [key, "!!!RNStyleSheet.hairlineWidth!!!"];
    }
    if (value.startsWith("platformColor(")) {
        const result = value.match(/platformColor\((.+)\)/);
        if (!result)
            return [key, "!!!undefined!!!"];
        const variables = result[1]
            .split(/[ ,]+/)
            .filter(Boolean)
            .map((v) => `'${v}'`);
        return [key, `!!!RNPlatformColor(${variables.join(",")})!!!`];
    }
    if (value.startsWith("platform(")) {
        const result = value.match(/platform\((.+)\)/);
        if (!result)
            return [key, "!!!undefined!!!"];
        const props = result[1]
            .trim()
            .split(/\s+/)
            .map((a) => {
            // Edge case:
            //   platform(android:platformColor(@android:color/holo_blue_bright))
            // Sometimes the value can has a colon, so we need to collect all values
            // and join them
            let [platform, ...values] = a.split(":");
            if (!values) {
                values = [platform];
                platform = "default";
            }
            const [key, value] = postCSSReplacer(platform, `${values.join("")}`);
            return `"${key}": ${value}`;
        });
        return [key, `!!!RNPlatform.select({ ${props} })!!!`];
    }
    return [key, value];
}
